from __future__ import annotations

from . import __version__, core
from .util import console, includes

HELP_MESSAGE = """\
Alternative ls command.

[bold]Usage:[/bold]
  lssr
  lssr path/to/target/dir
  lssr /absolute/path/to/target/dir

[bold]Options:[/bold]
  [blue]-r, --reverse[/blue]   Reverse the order of the sort.
  [blue]-t[/blue]              Sort by descending time modified (most recently modified first).
  [blue]-S[/blue]              Sort by size (largest file first).

[bold]Global options:[/bold]
  [blue]-h, --help[/blue]      Show this help message and exit.
  [blue]-V, --version[/blue]   Show program's version number and exit.

See https://github.com/seijinrosen/lssr for more information.\
"""


def print_help_message() -> None:
    console.print(HELP_MESSAGE)


def print_version() -> None:
    print(__version__)


def main(args: list[str]) -> None:
    if includes(args, {"-h", "--help"}):
        print_help_message()
        return

    if includes(args, {"-V", "--version"}):
        print_version()
        return

    core.main(args)



from __future__ import annotations

import sys
from dataclasses import dataclass
from enum import Enum, auto
from pathlib import Path

from rich import box
from rich.table import Table

from .util import console, ts2dt

# IGNORED_ITEMS = {".DS_Store"}


class SortMode(Enum):
    DEFAULT = auto()
    MTIME = auto()
    SIZE = auto()


@dataclass
class Options:
    reverse: bool
    sort_mode: SortMode


def get_single_hyphen_options(args: list[str]) -> list[str]:
    ret: list[str] = []
    for x in args:
        if not x.startswith("-"):
            continue
        if x.startswith("--"):
            continue
        ret += x.lstrip("-")
    return ret


def get_sort_mode(options: list[str]) -> SortMode:
    for x in options[::-1]:
        if x == "t":
            return SortMode.MTIME
        elif x == "S":
            return SortMode.SIZE
    return SortMode.DEFAULT


def get_options(args: list[str]) -> Options:
    single_hyphen_options = get_single_hyphen_options(args)
    double_hyphen_options = [x.lstrip("--") for x in args if x.startswith("--")]
    return Options(
        reverse="r" in single_hyphen_options or "reverse" in double_hyphen_options,
        sort_mode=get_sort_mode(single_hyphen_options),
    )


def get_info_message(items: list[Path]) -> str:
    dir_count = sum(p.is_dir() for p in items)
    file_count = len(items) - dir_count
    return f"{len(items)} items ({dir_count} dirs, {file_count} files)"


def colored(p: Path) -> str:
    # if p.name in IGNORED_ITEMS:
    #     return "[dim]"
    if p.is_dir():
        return "[blue]"
    return ""


def create_table(items: list[Path]) -> Table:
    table = Table(box=box.MINIMAL_DOUBLE_HEAD)
    table.add_column(justify="right")
    table.add_column("Name")
    table.add_column("Last updated")
    table.add_column("Size", justify="right")
    for i, p in enumerate(items, start=1):
        table.add_row(
            str(i),
            colored(p) + p.name,
            ts2dt(p.stat().st_mtime),
            f"{p.stat().st_size:,}",
        )
    return table


def get_sorted_items(
    target_path: Path,
    reverse: bool = False,
    sort_mode: SortMode = SortMode.DEFAULT,
) -> list[Path]:
    def get_sort_key(p: Path):
        if sort_mode == SortMode.DEFAULT:
            return (p.is_file(), p.name)
        if sort_mode == SortMode.MTIME:
            return (-p.stat().st_mtime_ns, p.is_file(), p.name)
        if sort_mode == SortMode.SIZE:
            return (-p.stat().st_size, p.is_file(), p.name)

    return sorted(target_path.iterdir(), key=get_sort_key, reverse=reverse)


def get_target_strpath(args: list[str]) -> str:
    not_option_args = [x for x in args if not x.startswith("-")]
    if not not_option_args:
        return "."
    return not_option_args[0]


def main(args: list[str]) -> None:
    target_path = Path(get_target_strpath(args))
    options = get_options(args)

    if not target_path.exists():
        sys.exit(f"{target_path}: No such file or directory")

    if target_path.is_file():
        console.print(target_path.name + " is file.")
        console.print(create_table([target_path]))
        return

    sorted_items = get_sorted_items(
        target_path, reverse=options.reverse, sort_mode=options.sort_mode
    )
    console.print(get_info_message(sorted_items))
    console.print(create_table(sorted_items))



from __future__ import annotations

from datetime import datetime

from rich.console import Console

console = Console()


def ts2dt(timestamp: float) -> str:
    dt = datetime.fromtimestamp(timestamp)
    return (
        "[bold cyan]"
        + str(dt.date())
        + " [bright_green]"
        + dt.time().strftime("%H:%M:%S")
    )


def includes(args: list[str], flags: set[str]) -> bool:
    return any(s in args for s in flags)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['lssr']

package_data = \
{'': ['*']}

install_requires = \
['rich>=12.4.4,<13.0.0']

entry_points = \
{'console_scripts': ['lssr = lssr.__main__:main']}

setup_kwargs = {
    'name': 'lssr',
    'version': '1.0.0',
    'description': 'Alternative ls command.',
    'long_description': '# lssr\n\nAlternative ls command.\n\n[![PyPI](https://img.shields.io/pypi/v/lssr)](https://pypi.python.org/pypi/lssr)\n[![PyPI - Python Version](https://img.shields.io/pypi/pyversions/lssr)](https://pypi.python.org/pypi/lssr)\n[![Tests](https://github.com/seijinrosen/lssr/actions/workflows/tests.yml/badge.svg)](https://github.com/seijinrosen/lssr/actions/workflows/tests.yml)\n[![CodeQL](https://github.com/seijinrosen/lssr/actions/workflows/codeql-analysis.yml/badge.svg)](https://github.com/seijinrosen/lssr/actions/workflows/codeql-analysis.yml)\n[![codecov](https://codecov.io/gh/seijinrosen/lssr/branch/main/graph/badge.svg)](https://codecov.io/gh/seijinrosen/lssr)\n[![Downloads](https://pepy.tech/badge/lssr)](https://pepy.tech/project/lssr)\n[![Downloads](https://pepy.tech/badge/lssr/month)](https://pepy.tech/project/lssr)\n[![Downloads](https://pepy.tech/badge/lssr/week)](https://pepy.tech/project/lssr)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n\n## インストール\n\nPython 3.7 以上がインストールされていれば利用可能です。\n\n```sh\npip install lssr\n```\n\n## 使い方\n\n```sh\n# カレントディレクトリにあるアイテムを表示\nlssr\n\n# 指定したディレクトリにあるアイテムを表示（相対パス）\nlssr path/to/target/dir\n\n# 絶対パスでの指定\nlssr /absolute/path/to/target/dir\n\n# ソート順を逆にする\nlssr -r\nlssr --reverse path/to/target/dir\n\n# 最終内容更新時刻順の新しい順にソート\nlssr -t\n# 古い順\nlssr -tr\n\n# ファイルサイズの大きい順にソート\nlssr -S\n# 小さい順\nlssr -rS\n\n# オプションの並び順は、ある程度自由です\nlssr -t path/to/target/dir --reverse\n\n# ヘルプを表示\nlssr -h\nlssr --help\n\n# バージョンを表示\nlssr -V\nlssr --version\n```\n\n## `ls` コマンドとの違い\n\n- アイテムの並び順が異なります。デフォルトで、以下の順序でアイテムが表示されます。\n  - フォルダ -> ファイル\n  - Unicode\n\n  つまり、GitHub と同様の並び順になるはずです。\n\n- ドットファイルを含んだリスト形式でのカラー表示がデフォルトです（`ls -AGl` と同等）。\n- 現在、パスを2つ以上指定することはできません。その代わり、オプションの指定はパスの前後どちらでも良いです。\n- 多くのオプションがまだ実装されていません。利用できるオプションは上記「使い方」やヘルプコマンドを参照してください。\n',
    'author': 'seijinrosen',
    'author_email': '86702775+seijinrosen@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/seijinrosen',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
